@@ -1,6 +1,8 @@
 # flake8: NOQA: E501
 """Sphinx configuration for libtmux."""
 
+from __future__ import annotations
+
 import contextlib
 import inspect
 import pathlib
@@ -22,7 +24,7 @@ sys.path.insert(0, str(project_src))
 sys.path.insert(0, str(cwd / "_ext"))
 
 # package data
-about: t.Dict[str, str] = {}
+about: dict[str, str] = {}
 with (project_src / "libtmux" / "__about__.py").open() as fp:
     exec(fp.read(), about)
 
@@ -58,8 +60,8 @@ master_doc = "index"
 project = about["__title__"]
 project_copyright = about["__copyright__"]
 
-version = "%s" % (".".join(about["__version__"].split("."))[:2])
-release = "%s" % (about["__version__"])
+version = "{}".format(".".join(about["__version__"].split("."))[:2])
+release = "{}".format(about["__version__"])
 
 exclude_patterns = ["_build"]
 
@@ -71,8 +73,8 @@ html_static_path = ["_static"]
 html_css_files = ["css/custom.css"]
 html_extra_path = ["manifest.json"]
 html_theme = "furo"
-html_theme_path: t.List[str] = []
-html_theme_options: t.Dict[str, t.Union[str, t.List[t.Dict[str, str]]]] = {
+html_theme_path: list[str] = []
+html_theme_options: dict[str, str | list[dict[str, str]]] = {
     "light_logo": "img/libtmux.svg",
     "dark_logo": "img/libtmux.svg",
     "footer_icons": [
@@ -90,7 +92,7 @@ html_theme_options: t.Dict[str, t.Union[str, t.List[t.Dict[str, str]]]] = {
     "source_repository": f"{about['__github__']}/",
     "source_branch": "master",
     "source_directory": "docs/",
-    "announcement": "<em>Friendly reminder:</em> 📌 Pin the package, libtmux is pre-1.0 and APIs will be <a href='/migration.html'>changing</a> throughout 2022-2024.",
+    "announcement": "<em>Friendly reminder:</em> 📌 Pin the package, libtmux is pre-1.0 and APIs will be <a href='/migration.html'>changing</a> throughout 2025.",
 }
 html_sidebars = {
     "**": [
@@ -104,7 +106,7 @@ html_sidebars = {
 }
 
 # linkify_issues
-issue_url_tpl = f'{about["__github__"]}/issues/{{issue_id}}'
+issue_url_tpl = f"{about['__github__']}/issues/{{issue_id}}"
 
 # sphinx.ext.autodoc
 autoclass_content = "both"
@@ -133,12 +135,12 @@ ogp_image = "_static/img/icons/icon-192x192.png"
 ogp_site_name = about["__title__"]
 
 intersphinx_mapping = {
-    "": ("https://docs.python.org/", None),
+    "python": ("https://docs.python.org/", None),
     "pytest": ("https://docs.pytest.org/en/stable/", None),
 }
 
 
-def linkcode_resolve(domain: str, info: t.Dict[str, str]) -> t.Union[None, str]:
+def linkcode_resolve(domain: str, info: dict[str, str]) -> None | str:
     """
     Determine the URL corresponding to Python object.
 
@@ -186,7 +188,7 @@ def linkcode_resolve(domain: str, info: t.Dict[str, str]) -> t.Union[None, str]:
     except Exception:
         lineno = None
 
-    linespec = "#L%d-L%d" % (lineno, lineno + len(source) - 1) if lineno else ""
+    linespec = f"#L{lineno}-L{lineno + len(source) - 1}" if lineno else ""
 
     fn = relpath(fn, start=pathlib.Path(libtmux.__file__).parent)
 
@@ -208,7 +210,7 @@ def linkcode_resolve(domain: str, info: t.Dict[str, str]) -> t.Union[None, str]:
     )
 
 
-def remove_tabs_js(app: "Sphinx", exc: Exception) -> None:
+def remove_tabs_js(app: Sphinx, exc: Exception) -> None:
     """Remove tabs.js from _static after build."""
     # Fix for sphinx-inline-tabs#18
     if app.builder.format == "html" and not exc:
@@ -217,6 +219,6 @@ def remove_tabs_js(app: "Sphinx", exc: Exception) -> None:
             tabs_js.unlink()  # When python 3.7 deprecated, use missing_ok=True
 
 
-def setup(app: "Sphinx") -> None:
+def setup(app: Sphinx) -> None:
     """Configure Sphinx app hooks."""
     app.connect("build-finished", remove_tabs_js)
