@@ -5,20 +5,20 @@ libtmux.common
 
 """
 
+from __future__ import annotations
+
 import logging
 import re
 import shutil
 import subprocess
 import sys
 import typing as t
-from typing import Dict, Optional, Union
 
 from . import exc
-from ._compat import LooseVersion, console_to_str, str_from_console
+from ._compat import LooseVersion
 
 if t.TYPE_CHECKING:
-    pass
-
+    from collections.abc import Callable
 
 logger = logging.getLogger(__name__)
 
@@ -27,12 +27,45 @@ logger = logging.getLogger(__name__)
 TMUX_MIN_VERSION = "1.8"
 
 #: Most recent version of tmux supported
-TMUX_MAX_VERSION = "3.4"
+TMUX_MAX_VERSION = "3.6"
 
-SessionDict = t.Dict[str, t.Any]
-WindowDict = t.Dict[str, t.Any]
-WindowOptionDict = t.Dict[str, t.Any]
-PaneDict = t.Dict[str, t.Any]
+#: Minimum version before deprecation warning is shown
+TMUX_SOFT_MIN_VERSION = "3.2a"
+
+SessionDict = dict[str, t.Any]
+WindowDict = dict[str, t.Any]
+WindowOptionDict = dict[str, t.Any]
+PaneDict = dict[str, t.Any]
+
+#: Flag to ensure deprecation warning is only shown once per process
+_version_deprecation_checked: bool = False
+
+
+def _check_deprecated_version(version: LooseVersion) -> None:
+    """Check if tmux version is deprecated and warn once.
+
+    This is called from get_version() on first invocation.
+    """
+    global _version_deprecation_checked
+    if _version_deprecation_checked:
+        return
+    _version_deprecation_checked = True
+
+    import os
+    import warnings
+
+    if os.environ.get("LIBTMUX_SUPPRESS_VERSION_WARNING"):
+        return
+
+    if version < LooseVersion(TMUX_SOFT_MIN_VERSION):
+        warnings.warn(
+            f"tmux {version} is deprecated and will be unsupported in a future "
+            f"libtmux release. Please upgrade to tmux {TMUX_SOFT_MIN_VERSION} "
+            "or newer. Set LIBTMUX_SUPPRESS_VERSION_WARNING=1 to suppress this "
+            "warning.",
+            FutureWarning,
+            stacklevel=4,
+        )
 
 
 class EnvironmentMixin:
@@ -40,9 +73,9 @@ class EnvironmentMixin:
 
     _add_option = None
 
-    cmd: t.Callable[[t.Any, t.Any], "tmux_cmd"]
+    cmd: Callable[[t.Any, t.Any], tmux_cmd]
 
-    def __init__(self, add_option: Optional[str] = None) -> None:
+    def __init__(self, add_option: str | None = None) -> None:
         self._add_option = add_option
 
     def set_environment(self, name: str, value: str) -> None:
@@ -69,7 +102,8 @@ class EnvironmentMixin:
                 if isinstance(cmd.stderr, list) and len(cmd.stderr) == 1
                 else cmd.stderr
             )
-            raise ValueError("tmux set-environment stderr: %s" % cmd.stderr)
+            msg = f"tmux set-environment stderr: {cmd.stderr}"
+            raise ValueError(msg)
 
     def unset_environment(self, name: str) -> None:
         """Unset environment variable ``$ tmux set-environment -u <name>``.
@@ -92,7 +126,8 @@ class EnvironmentMixin:
                 if isinstance(cmd.stderr, list) and len(cmd.stderr) == 1
                 else cmd.stderr
             )
-            raise ValueError("tmux set-environment stderr: %s" % cmd.stderr)
+            msg = f"tmux set-environment stderr: {cmd.stderr}"
+            raise ValueError(msg)
 
     def remove_environment(self, name: str) -> None:
         """Remove environment variable ``$ tmux set-environment -r <name>``.
@@ -115,9 +150,10 @@ class EnvironmentMixin:
                 if isinstance(cmd.stderr, list) and len(cmd.stderr) == 1
                 else cmd.stderr
             )
-            raise ValueError("tmux set-environment stderr: %s" % cmd.stderr)
+            msg = f"tmux set-environment stderr: {cmd.stderr}"
+            raise ValueError(msg)
 
-    def show_environment(self) -> Dict[str, Union[bool, str]]:
+    def show_environment(self) -> dict[str, bool | str]:
         """Show environment ``$ tmux show-environment -t [session]``.
 
         Return dict of environment variables for the session.
@@ -138,7 +174,7 @@ class EnvironmentMixin:
         cmd = self.cmd(*tmux_args)
         output = cmd.stdout
         opts = [tuple(item.split("=", 1)) for item in output]
-        opts_dict: t.Dict[str, t.Union[str, bool]] = {}
+        opts_dict: dict[str, str | bool] = {}
         for _t in opts:
             if len(_t) == 2:
                 opts_dict[_t[0]] = _t[1]
@@ -149,7 +185,7 @@ class EnvironmentMixin:
 
         return opts_dict
 
-    def getenv(self, name: str) -> Optional[t.Union[str, bool]]:
+    def getenv(self, name: str) -> str | bool | None:
         """Show environment variable ``$ tmux show-environment -t [session] <name>``.
 
         Return the value of a specific variable if the name is specified.
@@ -166,7 +202,7 @@ class EnvironmentMixin:
         str
             Value of environment variable
         """
-        tmux_args: t.Tuple[t.Union[str, int], ...] = ()
+        tmux_args: tuple[str | int, ...] = ()
 
         tmux_args += ("show-environment",)
         if self._add_option:
@@ -175,7 +211,7 @@ class EnvironmentMixin:
         cmd = self.cmd(*tmux_args)
         output = cmd.stdout
         opts = [tuple(item.split("=", 1)) for item in output]
-        opts_dict: t.Dict[str, t.Union[str, bool]] = {}
+        opts_dict: dict[str, str | bool] = {}
         for _t in opts:
             if len(_t) == 2:
                 opts_dict[_t[0]] = _t[1]
@@ -219,11 +255,11 @@ class tmux_cmd:
     def __init__(self, *args: t.Any) -> None:
         tmux_bin = shutil.which("tmux")
         if not tmux_bin:
-            raise exc.TmuxCommandNotFound()
+            raise exc.TmuxCommandNotFound
 
         cmd = [tmux_bin]
         cmd += args  # add the command arguments to cmd
-        cmd = [str_from_console(c) for c in cmd]
+        cmd = [str(c) for c in cmd]
 
         self.cmd = cmd
 
@@ -232,6 +268,8 @@ class tmux_cmd:
                 cmd,
                 stdout=subprocess.PIPE,
                 stderr=subprocess.PIPE,
+                text=True,
+                errors="backslashreplace",
             )
             stdout, stderr = self.process.communicate()
             returncode = self.process.returncode
@@ -241,14 +279,12 @@ class tmux_cmd:
 
         self.returncode = returncode
 
-        stdout_str = console_to_str(stdout)
-        stdout_split = stdout_str.split("\n")
+        stdout_split = stdout.split("\n")
         # remove trailing newlines from stdout
         while stdout_split and stdout_split[-1] == "":
             stdout_split.pop()
 
-        stderr_str = console_to_str(stderr)
-        stderr_split = stderr_str.split("\n")
+        stderr_split = stderr.split("\n")
         self.stderr = list(filter(None, stderr_split))  # filter empty values
 
         if "has-session" in cmd and len(self.stderr) and not stdout_split:
@@ -282,10 +318,13 @@ def get_version() -> LooseVersion:
     if proc.stderr:
         if proc.stderr[0] == "tmux: unknown option -- V":
             if sys.platform.startswith("openbsd"):  # openbsd has no tmux -V
-                return LooseVersion("%s-openbsd" % TMUX_MAX_VERSION)
+                return LooseVersion(f"{TMUX_MAX_VERSION}-openbsd")
+            msg = (
+                f"libtmux supports tmux {TMUX_MIN_VERSION} and greater. This system"
+                " is running tmux 1.3 or earlier."
+            )
             raise exc.LibTmuxException(
-                "libtmux supports tmux %s and greater. This system"
-                " is running tmux 1.3 or earlier." % TMUX_MIN_VERSION,
+                msg,
             )
         raise exc.VersionTooLow(proc.stderr)
 
@@ -293,11 +332,13 @@ def get_version() -> LooseVersion:
 
     # Allow latest tmux HEAD
     if version == "master":
-        return LooseVersion("%s-master" % TMUX_MAX_VERSION)
+        return LooseVersion(f"{TMUX_MAX_VERSION}-master")
 
     version = re.sub(r"[a-z-]", "", version)
 
-    return LooseVersion(version)
+    version_obj = LooseVersion(version)
+    _check_deprecated_version(version_obj)
+    return version_obj
 
 
 def has_version(version: str) -> bool:
@@ -412,16 +453,15 @@ def has_minimum_version(raises: bool = True) -> bool:
         if raises:
             msg = (
                 f"libtmux only supports tmux {TMUX_MIN_VERSION} and greater. This "
-                + f"system has {get_version()} installed. Upgrade your tmux to use "
-                + "libtmux."
+                f"system has {get_version()} installed. Upgrade your tmux to use "
+                "libtmux."
             )
             raise exc.VersionTooLow(msg)
-        else:
-            return False
+        return False
     return True
 
 
-def session_check_name(session_name: t.Optional[str]) -> None:
+def session_check_name(session_name: str | None) -> None:
     """Raise exception session name invalid, modeled after tmux function.
 
     tmux(1) session names may not be empty, or include periods or colons.
@@ -439,13 +479,13 @@ def session_check_name(session_name: t.Optional[str]) -> None:
     """
     if session_name is None or len(session_name) == 0:
         raise exc.BadSessionName(reason="empty", session_name=session_name)
-    elif "." in session_name:
+    if "." in session_name:
         raise exc.BadSessionName(reason="contains periods", session_name=session_name)
-    elif ":" in session_name:
+    if ":" in session_name:
         raise exc.BadSessionName(reason="contains colons", session_name=session_name)
 
 
-def handle_option_error(error: str) -> t.Type[exc.OptionError]:
+def handle_option_error(error: str) -> type[exc.OptionError]:
     """Raise exception if error in option command found.
 
     In tmux 3.0, show-option and show-window-option return invalid option instead of
@@ -474,12 +514,11 @@ def handle_option_error(error: str) -> t.Type[exc.OptionError]:
     """
     if "unknown option" in error:
         raise exc.UnknownOption(error)
-    elif "invalid option" in error:
+    if "invalid option" in error:
         raise exc.InvalidOption(error)
-    elif "ambiguous option" in error:
+    if "ambiguous option" in error:
         raise exc.AmbiguousOption(error)
-    else:
-        raise exc.OptionError(error)  # Raise generic option error
+    raise exc.OptionError(error)  # Raise generic option error
 
 
 def get_libtmux_version() -> LooseVersion:
