@@ -1,16 +1,24 @@
+"""libtmux pytest plugin."""
+
+from __future__ import annotations
+
+import contextlib
+import functools
 import getpass
 import logging
 import os
-import pathlib
 import typing as t
 
 import pytest
 
 from libtmux import exc
 from libtmux.server import Server
-from libtmux.test import TEST_SESSION_PREFIX, get_test_session_name, namer
+from libtmux.test.constants import TEST_SESSION_PREFIX
+from libtmux.test.random import get_test_session_name, namer
 
 if t.TYPE_CHECKING:
+    import pathlib
+
     from libtmux.session import Session
 
 logger = logging.getLogger(__name__)
@@ -25,13 +33,13 @@ def home_path(tmp_path_factory: pytest.TempPathFactory) -> pathlib.Path:
 
 @pytest.fixture(scope="session")
 def home_user_name() -> str:
-    """Default username to set for :func:`user_path` fixture."""
+    """Return default username to set for :func:`user_path` fixture."""
     return getpass.getuser()
 
 
 @pytest.fixture(scope="session")
 def user_path(home_path: pathlib.Path, home_user_name: str) -> pathlib.Path:
-    """Default temporary user directory.
+    """Ensure and return temporary user directory.
 
     Used by: :func:`config_file`, :func:`zshrc`
 
@@ -45,7 +53,7 @@ def user_path(home_path: pathlib.Path, home_user_name: str) -> pathlib.Path:
 @pytest.mark.skipif(USING_ZSH, reason="Using ZSH")
 @pytest.fixture(scope="session")
 def zshrc(user_path: pathlib.Path) -> pathlib.Path:
-    """This quiets ZSH default message.
+    """Suppress ZSH default message.
 
     Needs a startup file .zshenv, .zprofile, .zshrc, .zlogin.
     """
@@ -56,7 +64,7 @@ def zshrc(user_path: pathlib.Path) -> pathlib.Path:
 
 @pytest.fixture(scope="session")
 def config_file(user_path: pathlib.Path) -> pathlib.Path:
-    """Default `.tmux.conf` configuration.
+    """Return fixture for ``.tmux.conf`` configuration.
 
     - ``base-index -g 1``
 
@@ -80,7 +88,7 @@ def clear_env(monkeypatch: pytest.MonkeyPatch) -> None:
 
     tmux show-environment tests were being interrupted due to a lot of crazy env vars.
     """
-    for k, v in os.environ.items():
+    for k in os.environ:
         if not any(
             needle in k.lower()
             for needle in [
@@ -102,13 +110,13 @@ def clear_env(monkeypatch: pytest.MonkeyPatch) -> None:
             monkeypatch.delenv(k)
 
 
-@pytest.fixture(scope="function")
+@pytest.fixture
 def server(
     request: pytest.FixtureRequest,
     monkeypatch: pytest.MonkeyPatch,
     config_file: pathlib.Path,
 ) -> Server:
-    """Returns a new, temporary :class:`libtmux.Server`
+    """Return new, temporary :class:`libtmux.Server`.
 
     >>> from libtmux.server import Server
 
@@ -133,19 +141,61 @@ def server(
 
         >>> result.assert_outcomes(passed=1)
     """
-    t = Server(socket_name="libtmux_test%s" % next(namer))
+    server = Server(socket_name=f"libtmux_test{next(namer)}")
 
     def fin() -> None:
-        t.kill_server()
+        server.kill()
 
     request.addfinalizer(fin)
 
-    return t
+    return server
 
 
-@pytest.fixture(scope="function")
-def session(request: pytest.FixtureRequest, server: Server) -> "Session":
-    """Returns a new, temporary :class:`libtmux.Session`
+@pytest.fixture
+def session_params() -> dict[str, t.Any]:
+    """Return new, temporary :class:`libtmux.Session`.
+
+    >>> import pytest
+    >>> from libtmux.session import Session
+
+    >>> @pytest.fixture
+    ... def session_params(session_params):
+    ...     return {
+    ...         'x': 800,
+    ...         'y': 600,
+    ...     }
+
+    >>> def test_example(session: "Session") -> None:
+    ...     assert isinstance(session.name, str)
+    ...     assert session.name.startswith('libtmux_')
+    ...     window = session.new_window(window_name='new one')
+    ...     assert window.name == 'new one'
+
+    .. ::
+        >>> locals().keys()
+        dict_keys(...)
+
+        >>> source = ''.join([e.source for e in request._pyfuncitem.dtest.examples][:4])
+        >>> pytester = request.getfixturevalue('pytester')
+
+        >>> pytester.makepyfile(**{'whatever.py': source})
+        PosixPath(...)
+
+        >>> result = pytester.runpytest('whatever.py', '--disable-warnings')
+        ===...
+
+        >>> result.assert_outcomes(passed=1)
+    """
+    return {}
+
+
+@pytest.fixture
+def session(
+    request: pytest.FixtureRequest,
+    session_params: dict[str, t.Any],
+    server: Server,
+) -> Session:
+    """Return new, temporary :class:`libtmux.Session`.
 
     >>> from libtmux.session import Session
 
@@ -173,7 +223,9 @@ def session(request: pytest.FixtureRequest, server: Server) -> "Session":
     session_name = "tmuxp"
 
     if not server.has_session(session_name):
-        server.cmd("new-session", "-d", "-s", session_name)
+        server.new_session(
+            session_name=session_name,
+        )
 
     # find current sessions prefixed with tmuxp
     old_test_sessions = []
@@ -184,10 +236,10 @@ def session(request: pytest.FixtureRequest, server: Server) -> "Session":
 
     TEST_SESSION_NAME = get_test_session_name(server=server)
 
-    try:
-        session = server.new_session(session_name=TEST_SESSION_NAME)
-    except exc.LibTmuxException as e:
-        raise e
+    session = server.new_session(
+        session_name=TEST_SESSION_NAME,
+        **session_params,
+    )
 
     """
     Make sure that tmuxp can :ref:`test_builder_visually` and switches to
@@ -196,16 +248,68 @@ def session(request: pytest.FixtureRequest, server: Server) -> "Session":
     session_id = session.session_id
     assert session_id is not None
 
-    try:
+    with contextlib.suppress(exc.LibTmuxException):
         server.switch_client(target_session=session_id)
-    except exc.LibTmuxException:
-        # server.attach_session(session.get('session_id'))
-        pass
 
     for old_test_session in old_test_sessions:
-        logger.debug(f"Old test test session {old_test_session} found. Killing it.")
+        logger.debug("Old test test session %s found. Killing it.", old_test_session)
         server.kill_session(old_test_session)
-    assert TEST_SESSION_NAME == session.session_name
+    assert session.session_name == TEST_SESSION_NAME
     assert TEST_SESSION_NAME != "tmuxp"
 
     return session
+
+
+@pytest.fixture
+def TestServer(
+    request: pytest.FixtureRequest,
+) -> type[Server]:
+    """Create a temporary tmux server that cleans up after itself.
+
+    This is similar to the server pytest fixture, but can be used outside of pytest.
+    The server will be killed when the test completes.
+
+    Returns
+    -------
+    type[Server]
+        A factory function that returns a Server with a unique socket_name
+
+    Examples
+    --------
+    >>> server = Server()  # Create server instance
+    >>> server.new_session()
+    Session($... ...)
+    >>> server.is_alive()
+    True
+    >>> # Each call creates a new server with unique socket
+    >>> server2 = Server()
+    >>> server2.socket_name != server.socket_name
+    True
+    """
+    created_sockets: list[str] = []
+
+    def on_init(server: Server) -> None:
+        """Track created servers for cleanup."""
+        created_sockets.append(server.socket_name or "default")
+
+    def socket_name_factory() -> str:
+        """Generate unique socket names."""
+        return f"libtmux_test{next(namer)}"
+
+    def fin() -> None:
+        """Kill all servers created with these sockets."""
+        for socket_name in created_sockets:
+            server = Server(socket_name=socket_name)
+            if server.is_alive():
+                server.kill()
+
+    request.addfinalizer(fin)
+
+    return t.cast(
+        "type[Server]",
+        functools.partial(
+            Server,
+            on_init=on_init,
+            socket_name_factory=socket_name_factory,
+        ),
+    )
