@@ -4,6 +4,9 @@ libtmux.server
 ~~~~~~~~~~~~~~
 
 """
+
+from __future__ import annotations
+
 import logging
 import os
 import pathlib
@@ -12,6 +15,7 @@ import subprocess
 import typing as t
 import warnings
 
+from libtmux import exc, formats
 from libtmux._internal.query_list import QueryList
 from libtmux.common import tmux_cmd
 from libtmux.neo import fetch_objs
@@ -19,7 +23,6 @@ from libtmux.pane import Pane
 from libtmux.session import Session
 from libtmux.window import Window
 
-from . import exc, formats
 from .common import (
     EnvironmentMixin,
     PaneDict,
@@ -30,7 +33,12 @@ from .common import (
 )
 
 if t.TYPE_CHECKING:
-    from typing_extensions import TypeAlias
+    import types
+    from typing import TypeAlias
+
+    from typing_extensions import Self
+
+    from libtmux._internal.types import StrPath
 
     DashLiteral: TypeAlias = t.Literal["-"]
 
@@ -56,6 +64,8 @@ class Server(EnvironmentMixin):
     socket_path : str, optional
     config_file : str, optional
     colors : str, optional
+    on_init : callable, optional
+    socket_name_factory : callable, optional
 
     Examples
     --------
@@ -66,14 +76,21 @@ class Server(EnvironmentMixin):
     [Session($1 ...)]
 
     >>> server.sessions[0].windows
-    [Window(@1 1:..., Session($1 ...)]
+    [Window(@1 1:..., Session($1 ...))]
 
-    >>> server.sessions[0].attached_window
+    >>> server.sessions[0].active_window
     Window(@1 1:..., Session($1 ...))
 
-    >>> server.sessions[0].attached_pane
+    >>> server.sessions[0].active_pane
     Pane(%1 Window(@1 1:..., Session($1 ...)))
 
+    The server can be used as a context manager to ensure proper cleanup:
+
+    >>> with Server() as server:
+    ...     session = server.new_session()
+    ...     # Do work with the session
+    ...     # Server will be killed automatically when exiting the context
+
     References
     ----------
     .. [server_manual] CLIENTS AND SESSIONS. openbsd manpage for TMUX(1)
@@ -95,7 +112,7 @@ class Server(EnvironmentMixin):
     config_file = None
     """Passthrough to ``[-f file]``"""
     colors = None
-    """``-2`` or ``-8``"""
+    """``256`` or ``88``"""
     child_id_attribute = "session_id"
     """Unique child ID used by :class:`~libtmux.common.TmuxRelationalObject`"""
     formatter_prefix = "server_"
@@ -103,20 +120,24 @@ class Server(EnvironmentMixin):
 
     def __init__(
         self,
-        socket_name: t.Optional[str] = None,
-        socket_path: t.Optional[t.Union[str, pathlib.Path]] = None,
-        config_file: t.Optional[str] = None,
-        colors: t.Optional[int] = None,
+        socket_name: str | None = None,
+        socket_path: str | pathlib.Path | None = None,
+        config_file: str | None = None,
+        colors: int | None = None,
+        on_init: t.Callable[[Server], None] | None = None,
+        socket_name_factory: t.Callable[[], str] | None = None,
         **kwargs: t.Any,
     ) -> None:
         EnvironmentMixin.__init__(self, "-g")
-        self._windows: t.List[WindowDict] = []
-        self._panes: t.List[PaneDict] = []
+        self._windows: list[WindowDict] = []
+        self._panes: list[PaneDict] = []
 
         if socket_path is not None:
             self.socket_path = socket_path
         elif socket_name is not None:
             self.socket_name = socket_name
+        elif socket_name_factory is not None:
+            self.socket_name = socket_name_factory()
 
         tmux_tmpdir = pathlib.Path(os.getenv("TMUX_TMPDIR", "/tmp"))
         socket_name = self.socket_name or "default"
@@ -134,6 +155,39 @@ class Server(EnvironmentMixin):
         if colors:
             self.colors = colors
 
+        if on_init is not None:
+            on_init(self)
+
+    def __enter__(self) -> Self:
+        """Enter the context, returning self.
+
+        Returns
+        -------
+        :class:`Server`
+            The server instance
+        """
+        return self
+
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        exc_tb: types.TracebackType | None,
+    ) -> None:
+        """Exit the context, killing the server if it exists.
+
+        Parameters
+        ----------
+        exc_type : type[BaseException] | None
+            The type of the exception that was raised
+        exc_value : BaseException | None
+            The instance of the exception that was raised
+        exc_tb : types.TracebackType | None
+            The traceback of the exception that was raised
+        """
+        if self.is_alive():
+            self.kill()
+
     def is_alive(self) -> bool:
         """Return True if tmux server alive.
 
@@ -158,9 +212,9 @@ class Server(EnvironmentMixin):
         """
         tmux_bin = shutil.which("tmux")
         if tmux_bin is None:
-            raise exc.TmuxCommandNotFound()
+            raise exc.TmuxCommandNotFound
 
-        cmd_args: t.List[str] = ["list-sessions"]
+        cmd_args: list[str] = ["list-sessions"]
         if self.socket_name:
             cmd_args.insert(0, f"-L{self.socket_name}")
         if self.socket_path:
@@ -173,14 +227,49 @@ class Server(EnvironmentMixin):
     #
     # Command
     #
-    def cmd(self, *args: t.Any, **kwargs: t.Any) -> tmux_cmd:
-        """Execute tmux command, rsepective of socket name and file, return output.
+    def cmd(
+        self,
+        cmd: str,
+        *args: t.Any,
+        target: str | int | None = None,
+    ) -> tmux_cmd:
+        """Execute tmux command respective of socket name and file, return output.
 
         Examples
         --------
         >>> server.cmd('display-message', 'hi')
         <libtmux.common.tmux_cmd object at ...>
 
+        New session:
+
+        >>> server.cmd('new-session', '-d', '-P', '-F#{session_id}').stdout[0]
+        '$2'
+
+        >>> session.cmd('new-window', '-P').stdout[0]
+        'libtmux...:2.0'
+
+        Output of `tmux -L ... new-window -P -F#{window_id}` to a `Window` object:
+
+        >>> Window.from_window_id(window_id=session.cmd(
+        ... 'new-window', '-P', '-F#{window_id}').stdout[0], server=session.server)
+        Window(@4 3:..., Session($1 libtmux_...))
+
+        Create a pane from a window:
+
+        >>> window.cmd('split-window', '-P', '-F#{pane_id}').stdout[0]
+        '%5'
+
+        Output of `tmux -L ... split-window -P -F#{pane_id}` to a `Pane` object:
+
+        >>> Pane.from_pane_id(pane_id=window.cmd(
+        ... 'split-window', '-P', '-F#{pane_id}').stdout[0], server=window.server)
+        Pane(%... Window(@... ...:..., Session($1 libtmux_...)))
+
+        Parameters
+        ----------
+        target : str, optional
+            Optional custom target.
+
         Returns
         -------
         :class:`common.tmux_cmd`
@@ -191,25 +280,28 @@ class Server(EnvironmentMixin):
 
             Renamed from ``.tmux`` to ``.cmd``.
         """
-        cmd_args: t.List[t.Union[str, int]] = list(args)
+        svr_args: list[str | int] = [cmd]
+        cmd_args: list[str | int] = []
         if self.socket_name:
-            cmd_args.insert(0, f"-L{self.socket_name}")
+            svr_args.insert(0, f"-L{self.socket_name}")
         if self.socket_path:
-            cmd_args.insert(0, f"-S{self.socket_path}")
+            svr_args.insert(0, f"-S{self.socket_path}")
         if self.config_file:
-            cmd_args.insert(0, f"-f{self.config_file}")
+            svr_args.insert(0, f"-f{self.config_file}")
         if self.colors:
             if self.colors == 256:
-                cmd_args.insert(0, "-2")
+                svr_args.insert(0, "-2")
             elif self.colors == 88:
-                cmd_args.insert(0, "-8")
+                svr_args.insert(0, "-8")
             else:
-                raise exc.UnknownColorOption()
+                raise exc.UnknownColorOption
 
-        return tmux_cmd(*cmd_args, **kwargs)
+        cmd_args = ["-t", str(target), *args] if target is not None else [*args]
+
+        return tmux_cmd(*svr_args, *cmd_args)
 
     @property
-    def attached_sessions(self) -> t.List[Session]:
+    def attached_sessions(self) -> list[Session]:
         """Return active :class:`Session`s.
 
         Examples
@@ -221,22 +313,7 @@ class Server(EnvironmentMixin):
         -------
         list of :class:`Session`
         """
-        try:
-            sessions = self.sessions
-            attached_sessions = []
-
-            for session in sessions:
-                attached = session.session_attached
-                # for now session_active is a unicode
-                if attached != "0":
-                    logger.debug(f"session {session.name} attached")
-                    attached_sessions.append(session)
-                else:
-                    continue
-
-        except Exception:
-            return []
-        return attached_sessions
+        return self.sessions.filter(session_attached__noeq="1")
 
     def has_session(self, target_session: str, exact: bool = True) -> bool:
         """Return True if session exists.
@@ -263,18 +340,31 @@ class Server(EnvironmentMixin):
         if exact and has_gte_version("2.1"):
             target_session = f"={target_session}"
 
-        proc = self.cmd("has-session", "-t%s" % target_session)
+        proc = self.cmd("has-session", target=target_session)
 
-        if not proc.returncode:
-            return True
+        return bool(not proc.returncode)
 
-        return False
+    def kill(self) -> None:
+        """Kill tmux server.
 
-    def kill_server(self) -> None:
-        """Kill tmux server."""
+        >>> svr = Server(socket_name="testing")
+        >>> svr
+        Server(socket_name=testing)
+
+        >>> svr.new_session()
+        Session(...)
+
+        >>> svr.is_alive()
+        True
+
+        >>> svr.kill()
+
+        >>> svr.is_alive()
+        False
+        """
         self.cmd("kill-server")
 
-    def kill_session(self, target_session: t.Union[str, int]) -> "Server":
+    def kill_session(self, target_session: str | int) -> Server:
         """Kill tmux session.
 
         Parameters
@@ -291,7 +381,7 @@ class Server(EnvironmentMixin):
         ------
         :exc:`exc.BadSessionName`
         """
-        proc = self.cmd("kill-session", "-t%s" % target_session)
+        proc = self.cmd("kill-session", target=target_session)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
@@ -312,12 +402,12 @@ class Server(EnvironmentMixin):
         """
         session_check_name(target_session)
 
-        proc = self.cmd("switch-client", "-t%s" % target_session)
+        proc = self.cmd("switch-client", target=target_session)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
 
-    def attach_session(self, target_session: t.Optional[str] = None) -> None:
+    def attach_session(self, target_session: str | None = None) -> None:
         """Attach tmux session.
 
         Parameters
@@ -330,26 +420,22 @@ class Server(EnvironmentMixin):
         :exc:`exc.BadSessionName`
         """
         session_check_name(target_session)
-
-        tmux_args: t.Tuple[str, ...] = ()
-        if target_session:
-            tmux_args += ("-t%s" % target_session,)
-
-        proc = self.cmd("attach-session", *tmux_args)
+        proc = self.cmd("attach-session", target=target_session)
 
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
 
     def new_session(
         self,
-        session_name: t.Optional[str] = None,
+        session_name: str | None = None,
         kill_session: bool = False,
         attach: bool = False,
-        start_directory: t.Optional[str] = None,
-        window_name: t.Optional[str] = None,
-        window_command: t.Optional[str] = None,
-        x: t.Optional[t.Union[int, "DashLiteral"]] = None,
-        y: t.Optional[t.Union[int, "DashLiteral"]] = None,
+        start_directory: StrPath | None = None,
+        window_name: str | None = None,
+        window_command: str | None = None,
+        x: int | DashLiteral | None = None,
+        y: int | DashLiteral | None = None,
+        environment: dict[str, str] | None = None,
         *args: t.Any,
         **kwargs: t.Any,
     ) -> Session:
@@ -379,7 +465,7 @@ class Server(EnvironmentMixin):
         kill_session : bool, optional
             Kill current session if ``$ tmux has-session``.
             Useful for testing workspaces.
-        start_directory : str, optional
+        start_directory : str or PathLike, optional
             specifies the working directory in which the
             new session is created.
         window_name : str, optional
@@ -428,21 +514,22 @@ class Server(EnvironmentMixin):
 
             if self.has_session(session_name):
                 if kill_session:
-                    self.cmd("kill-session", "-t%s" % session_name)
-                    logger.info("session %s exists. killed it." % session_name)
+                    self.cmd("kill-session", target=session_name)
+                    logger.info("session %s exists. killed it.", session_name)
                 else:
+                    msg = f"Session named {session_name} exists"
                     raise exc.TmuxSessionExists(
-                        "Session named %s exists" % session_name,
+                        msg,
                     )
 
-        logger.debug(f"creating session {session_name}")
+        logger.debug("creating session %s", session_name)
 
         env = os.environ.get("TMUX")
 
         if env:
             del os.environ["TMUX"]
 
-        tmux_args: t.Tuple[t.Union[str, int], ...] = (
+        tmux_args: tuple[str | int, ...] = (
             "-P",
             "-F#{session_id}",  # output
         )
@@ -454,7 +541,9 @@ class Server(EnvironmentMixin):
             tmux_args += ("-d",)
 
         if start_directory:
-            tmux_args += ("-c", start_directory)
+            # as of 2014-02-08 tmux 1.9-dev doesn't expand ~ in new-session -c.
+            start_directory = pathlib.Path(start_directory).expanduser()
+            tmux_args += ("-c", str(start_directory))
 
         if window_name:
             tmux_args += ("-n", window_name)
@@ -465,6 +554,15 @@ class Server(EnvironmentMixin):
         if y is not None:
             tmux_args += ("-y", y)
 
+        if environment:
+            if has_gte_version("3.2"):
+                for k, v in environment.items():
+                    tmux_args += (f"-e{k}={v}",)
+            else:
+                logger.warning(
+                    "Environment flag ignored, tmux 3.2 or newer required.",
+                )
+
         if window_command:
             tmux_args += (window_command,)
 
@@ -479,7 +577,11 @@ class Server(EnvironmentMixin):
             os.environ["TMUX"] = env
 
         session_formatters = dict(
-            zip(["session_id"], session_stdout.split(formats.FORMAT_SEPARATOR)),
+            zip(
+                ["session_id"],
+                session_stdout.split(formats.FORMAT_SEPARATOR),
+                strict=False,
+            ),
         )
 
         return Session.from_session_id(
@@ -498,7 +600,7 @@ class Server(EnvironmentMixin):
         :meth:`.sessions.get() <libtmux._internal.query_list.QueryList.get()>` and
         :meth:`.sessions.filter() <libtmux._internal.query_list.QueryList.filter()>`
         """
-        sessions: t.List["Session"] = []
+        sessions: list[Session] = []
 
         try:
             for obj in fetch_objs(
@@ -519,7 +621,7 @@ class Server(EnvironmentMixin):
         :meth:`.windows.get() <libtmux._internal.query_list.QueryList.get()>` and
         :meth:`.windows.filter() <libtmux._internal.query_list.QueryList.filter()>`
         """
-        windows: t.List["Window"] = [
+        windows: list[Window] = [
             Window(server=self, **obj)
             for obj in fetch_objs(
                 list_cmd="list-windows",
@@ -538,11 +640,11 @@ class Server(EnvironmentMixin):
         :meth:`.panes.get() <libtmux._internal.query_list.QueryList.get()>` and
         :meth:`.panes.filter() <libtmux._internal.query_list.QueryList.filter()>`
         """
-        panes: t.List["Pane"] = [
+        panes: list[Pane] = [
             Pane(server=self, **obj)
             for obj in fetch_objs(
                 list_cmd="list-panes",
-                list_extra_args=["-s"],
+                list_extra_args=("-a",),
                 server=self,
             )
         ]
@@ -568,14 +670,33 @@ class Server(EnvironmentMixin):
                 f"{self.__class__.__name__}"
                 f"(socket_name={getattr(self, 'socket_name', 'default')})"
             )
-        elif self.socket_path is not None:
-            return f"{self.__class__.__name__}" f"(socket_path={self.socket_path})"
-        return f"{self.__class__.__name__}" f"(socket_path=/tmp/tmux-1000/default)"
+        if self.socket_path is not None:
+            return f"{self.__class__.__name__}(socket_path={self.socket_path})"
+        return (
+            f"{self.__class__.__name__}(socket_path=/tmp/tmux-{os.geteuid()}/default)"
+        )
 
     #
     # Legacy: Redundant stuff we want to remove
     #
-    def _list_panes(self) -> t.List[PaneDict]:
+    def kill_server(self) -> None:
+        """Kill tmux server.
+
+        Notes
+        -----
+        .. deprecated:: 0.30
+
+           Deprecated in favor of :meth:`.kill()`.
+
+        """
+        warnings.warn(
+            "Server.kill_server() is deprecated in favor of Server.kill()",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
+        self.cmd("kill-server")
+
+    def _list_panes(self) -> list[PaneDict]:
         """Return list of panes in :py:obj:`dict` form.
 
         Retrieved from ``$ tmux(1) list-panes`` stdout.
@@ -588,10 +709,14 @@ class Server(EnvironmentMixin):
            Deprecated in favor of :attr:`.panes`.
 
         """
-        warnings.warn("Server._list_panes() is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server._list_panes() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         return [p.__dict__ for p in self.panes]
 
-    def _update_panes(self) -> "Server":
+    def _update_panes(self) -> Server:
         """Update internal pane data and return ``self`` for chainability.
 
         .. deprecated:: 0.16
@@ -602,11 +727,15 @@ class Server(EnvironmentMixin):
         -------
         :class:`Server`
         """
-        warnings.warn("Server._update_panes() is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server._update_panes() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         self._list_panes()
         return self
 
-    def get_by_id(self, session_id: str) -> t.Optional[Session]:
+    def get_by_id(self, session_id: str) -> Session | None:
         """Return session by id. Deprecated in favor of :meth:`.sessions.get()`.
 
         .. deprecated:: 0.16
@@ -614,10 +743,14 @@ class Server(EnvironmentMixin):
            Deprecated by :meth:`.sessions.get()`.
 
         """
-        warnings.warn("Server.get_by_id() is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server.get_by_id() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         return self.sessions.get(session_id=session_id, default=None)
 
-    def where(self, kwargs: t.Dict[str, t.Any]) -> t.List[Session]:
+    def where(self, kwargs: dict[str, t.Any]) -> list[Session]:
         """Filter through sessions, return list of :class:`Session`.
 
         .. deprecated:: 0.16
@@ -625,13 +758,17 @@ class Server(EnvironmentMixin):
            Deprecated by :meth:`.session.filter()`.
 
         """
-        warnings.warn("Server.find_where() is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server.find_where() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         try:
             return self.sessions.filter(**kwargs)
         except IndexError:
             return []
 
-    def find_where(self, kwargs: t.Dict[str, t.Any]) -> t.Optional[Session]:
+    def find_where(self, kwargs: dict[str, t.Any]) -> Session | None:
         """Filter through sessions, return first :class:`Session`.
 
         .. deprecated:: 0.16
@@ -639,10 +776,14 @@ class Server(EnvironmentMixin):
            Slated to be removed in favor of :meth:`.sessions.get()`.
 
         """
-        warnings.warn("Server.find_where() is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server.find_where() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         return self.sessions.get(default=None, **kwargs)
 
-    def _list_windows(self) -> t.List[WindowDict]:
+    def _list_windows(self) -> list[WindowDict]:
         """Return list of windows in :py:obj:`dict` form.
 
         Retrieved from ``$ tmux(1) list-windows`` stdout.
@@ -655,10 +796,14 @@ class Server(EnvironmentMixin):
            Slated to be removed in favor of :attr:`.windows`.
 
         """
-        warnings.warn("Server._list_windows() is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server._list_windows() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         return [w.__dict__ for w in self.windows]
 
-    def _update_windows(self) -> "Server":
+    def _update_windows(self) -> Server:
         """Update internal window data and return ``self`` for chainability.
 
         .. deprecated:: 0.16
@@ -666,12 +811,16 @@ class Server(EnvironmentMixin):
            Deprecated in favor of :attr:`.windows` and returning ``self``.
 
         """
-        warnings.warn("Server._update_windows() is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server._update_windows() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         self._list_windows()
         return self
 
     @property
-    def _sessions(self) -> t.List[SessionDict]:
+    def _sessions(self) -> list[SessionDict]:
         """Property / alias to return :meth:`~._list_sessions`.
 
         .. deprecated:: 0.16
@@ -679,20 +828,28 @@ class Server(EnvironmentMixin):
            Slated to be removed in favor of :attr:`.sessions`.
 
         """
-        warnings.warn("Server._sessions is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server._sessions is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         return self._list_sessions()
 
-    def _list_sessions(self) -> t.List["SessionDict"]:
+    def _list_sessions(self) -> list[SessionDict]:
         """Return list of session object dictionaries.
 
         .. deprecated:: 0.16
 
            Slated to be removed in favor of :attr:`.sessions`.
         """
-        warnings.warn("Server._list_sessions() is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server._list_sessions() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         return [s.__dict__ for s in self.sessions]
 
-    def list_sessions(self) -> t.List[Session]:
+    def list_sessions(self) -> list[Session]:
         """Return list of :class:`Session` from the ``tmux(1)`` session.
 
         .. deprecated:: 0.16
@@ -703,11 +860,15 @@ class Server(EnvironmentMixin):
         -------
         list of :class:`Session`
         """
-        warnings.warn("Server.list_sessions is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server.list_sessions is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         return self.sessions
 
     @property
-    def children(self) -> QueryList["Session"]:
+    def children(self) -> QueryList[Session]:
         """Was used by TmuxRelationalObject (but that's longer used in this class).
 
         .. deprecated:: 0.16
@@ -715,5 +876,9 @@ class Server(EnvironmentMixin):
            Slated to be removed in favor of :attr:`.sessions`.
 
         """
-        warnings.warn("Server.children is deprecated", stacklevel=2)
+        warnings.warn(
+            "Server.children is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
+        )
         return self.sessions
