@@ -5,6 +5,8 @@ libtmux.window
 
 """
 
+from __future__ import annotations
+
 import dataclasses
 import logging
 import shlex
@@ -26,9 +28,20 @@ from . import exc
 from .common import PaneDict, WindowOptionDict, handle_option_error
 
 if t.TYPE_CHECKING:
+    import sys
+    import types
+
+    from libtmux._internal.types import StrPath
+
     from .server import Server
     from .session import Session
 
+    if sys.version_info >= (3, 11):
+        from typing import Self
+    else:
+        from typing_extensions import Self
+
+
 logger = logging.getLogger(__name__)
 
 
@@ -71,6 +84,13 @@ class Window(Obj):
     >>> window in session.windows
     True
 
+    The window can be used as a context manager to ensure proper cleanup:
+
+    >>> with session.new_window() as window:
+    ...     pane = window.split()
+    ...     # Do work with the pane
+    ...     # Window will be killed automatically when exiting the context
+
     References
     ----------
     .. [window_manual] tmux window. openbsd manpage for TMUX(1).
@@ -81,7 +101,40 @@ class Window(Obj):
        https://man.openbsd.org/tmux.1#DESCRIPTION. Accessed April 1st, 2018.
     """
 
-    server: "Server"
+    server: Server
+
+    def __enter__(self) -> Self:
+        """Enter the context, returning self.
+
+        Returns
+        -------
+        :class:`Window`
+            The window instance
+        """
+        return self
+
+    def __exit__(
+        self,
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        exc_tb: types.TracebackType | None,
+    ) -> None:
+        """Exit the context, killing the window if it exists.
+
+        Parameters
+        ----------
+        exc_type : type[BaseException] | None
+            The type of the exception that was raised
+        exc_value : BaseException | None
+            The instance of the exception that was raised
+        exc_tb : types.TracebackType | None
+            The traceback of the exception that was raised
+        """
+        if (
+            self.window_id is not None
+            and len(self.session.windows.filter(window_id=self.window_id)) > 0
+        ):
+            self.kill()
 
     def refresh(self) -> None:
         """Refresh window attributes from tmux."""
@@ -94,7 +147,7 @@ class Window(Obj):
         )
 
     @classmethod
-    def from_window_id(cls, server: "Server", window_id: str) -> "Window":
+    def from_window_id(cls, server: Server, window_id: str) -> Window:
         """Create Window from existing window_id."""
         window = fetch_obj(
             obj_key="window_id",
@@ -106,7 +159,7 @@ class Window(Obj):
         return cls(server=server, **window)
 
     @property
-    def session(self) -> "Session":
+    def session(self) -> Session:
         """Parent session of window."""
         assert isinstance(self.session_id, str)
         from libtmux.session import Session
@@ -114,14 +167,14 @@ class Window(Obj):
         return Session.from_session_id(server=self.server, session_id=self.session_id)
 
     @property
-    def panes(self) -> QueryList["Pane"]:
+    def panes(self) -> QueryList[Pane]:
         """Panes contained by window.
 
         Can be accessed via
         :meth:`.panes.get() <libtmux._internal.query_list.QueryList.get()>` and
         :meth:`.panes.filter() <libtmux._internal.query_list.QueryList.filter()>`
         """
-        panes: t.List["Pane"] = [
+        panes: list[Pane] = [
             Pane(server=self.server, **obj)
             for obj in fetch_objs(
                 list_cmd="list-panes",
@@ -141,7 +194,7 @@ class Window(Obj):
         self,
         cmd: str,
         *args: t.Any,
-        target: t.Optional[t.Union[str, int]] = None,
+        target: str | int | None = None,
     ) -> tmux_cmd:
         """Execute tmux subcommand within window context.
 
@@ -179,7 +232,7 @@ class Window(Obj):
     Commands (tmux-like)
     """
 
-    def select_pane(self, target_pane: t.Union[str, int]) -> t.Optional["Pane"]:
+    def select_pane(self, target_pane: str | int) -> Pane | None:
         """Select pane and return selected :class:`Pane`.
 
         ``$ tmux select-pane``.
@@ -193,7 +246,7 @@ class Window(Obj):
         -------
         :class:`Pane`
         """
-        if target_pane in ["-l", "-U", "-D", "-L", "-R"]:
+        if target_pane in {"-l", "-U", "-D", "-L", "-R"}:
             proc = self.cmd("select-pane", target_pane)
         else:
             proc = self.cmd("select-pane", target=target_pane)
@@ -206,16 +259,16 @@ class Window(Obj):
     def split(
         self,
         /,
-        target: t.Optional[t.Union[int, str]] = None,
-        start_directory: t.Optional[str] = None,
+        target: int | str | None = None,
+        start_directory: StrPath | None = None,
         attach: bool = False,
-        direction: t.Optional[PaneDirection] = None,
-        full_window_split: t.Optional[bool] = None,
-        zoom: t.Optional[bool] = None,
-        shell: t.Optional[str] = None,
-        size: t.Optional[t.Union[str, int]] = None,
-        environment: t.Optional[t.Dict[str, str]] = None,
-    ) -> "Pane":
+        direction: PaneDirection | None = None,
+        full_window_split: bool | None = None,
+        zoom: bool | None = None,
+        shell: str | None = None,
+        size: str | int | None = None,
+        environment: dict[str, str] | None = None,
+    ) -> Pane:
         """Split window on active pane and return the created :class:`Pane`.
 
         Parameters
@@ -223,7 +276,7 @@ class Window(Obj):
         attach : bool, optional
             make new window the current window after creating it, default
             True.
-        start_directory : str, optional
+        start_directory : str or PathLike, optional
             specifies the working directory in which the new window is created.
         direction : PaneDirection, optional
             split in direction. If none is specified, assume down.
@@ -260,15 +313,15 @@ class Window(Obj):
         self,
         /,
         # Adjustments
-        adjustment_direction: t.Optional[ResizeAdjustmentDirection] = None,
-        adjustment: t.Optional[int] = None,
+        adjustment_direction: ResizeAdjustmentDirection | None = None,
+        adjustment: int | None = None,
         # Manual
-        height: t.Optional[int] = None,
-        width: t.Optional[int] = None,
+        height: int | None = None,
+        width: int | None = None,
         # Expand / Shrink
-        expand: t.Optional[bool] = None,
-        shrink: t.Optional[bool] = None,
-    ) -> "Window":
+        expand: bool | None = None,
+        shrink: bool | None = None,
+    ) -> Window:
         """Resize tmux window.
 
         Parameters
@@ -308,18 +361,18 @@ class Window(Obj):
             warnings.warn("resize() requires tmux 2.9 or newer", stacklevel=2)
             return self
 
-        tmux_args: t.Tuple[str, ...] = ()
+        tmux_args: tuple[str, ...] = ()
 
-        ## Adjustments
+        # Adjustments
         if adjustment_direction:
             if adjustment is None:
-                raise exc.WindowAdjustmentDirectionRequiresAdjustment()
+                raise exc.WindowAdjustmentDirectionRequiresAdjustment
             tmux_args += (
                 f"{RESIZE_ADJUSTMENT_DIRECTION_FLAG_MAP[adjustment_direction]}",
                 str(adjustment),
             )
         elif height or width:
-            ## Manual resizing
+            # Manual resizing
             if height:
                 tmux_args += (f"-y{int(height)}",)
             if width:
@@ -338,11 +391,11 @@ class Window(Obj):
         self.refresh()
         return self
 
-    def last_pane(self) -> t.Optional["Pane"]:
+    def last_pane(self) -> Pane | None:
         """Return last pane."""
         return self.select_pane("-l")
 
-    def select_layout(self, layout: t.Optional[str] = None) -> "Window":
+    def select_layout(self, layout: str | None = None) -> Window:
         """Select layout for window.
 
         Wrapper for ``$ tmux select-layout <layout>``.
@@ -385,7 +438,7 @@ class Window(Obj):
 
         return self
 
-    def set_window_option(self, option: str, value: t.Union[int, str]) -> "Window":
+    def set_window_option(self, option: str, value: int | str) -> Window:
         """Set option for tmux window.
 
         Wraps ``$ tmux set-window-option <option> <value>``.
@@ -419,7 +472,7 @@ class Window(Obj):
 
         return self
 
-    def show_window_options(self, g: t.Optional[bool] = False) -> "WindowOptionDict":
+    def show_window_options(self, g: bool | None = False) -> WindowOptionDict:
         """Return dict of options for window.
 
         .. versionchanged:: 0.13.0
@@ -431,7 +484,7 @@ class Window(Obj):
         g : str, optional
             Pass ``-g`` flag for global variable, default False.
         """
-        tmux_args: t.Tuple[str, ...] = ()
+        tmux_args: tuple[str, ...] = ()
 
         if g:
             tmux_args += ("-g",)
@@ -445,12 +498,12 @@ class Window(Obj):
         # retaining quoted sub-strings.
         #   shlex.split('this is "a test"') => ['this', 'is', 'a test']
 
-        window_options: "WindowOptionDict" = {}
+        window_options: WindowOptionDict = {}
         for item in output:
             try:
                 key, val = shlex.split(item)
             except ValueError:
-                logger.exception(f"Error extracting option: {item}")
+                logger.exception("Error extracting option: %s", item)
             assert isinstance(key, str)
             assert isinstance(val, str)
 
@@ -463,7 +516,7 @@ class Window(Obj):
         self,
         option: str,
         g: bool = False,
-    ) -> t.Optional[t.Union[str, int]]:
+    ) -> str | int | None:
         """Return option value for the target window.
 
         todo: test and return True/False for on/off string
@@ -479,7 +532,7 @@ class Window(Obj):
         :exc:`exc.OptionError`, :exc:`exc.UnknownOption`,
         :exc:`exc.InvalidOption`, :exc:`exc.AmbiguousOption`
         """
-        tmux_args: t.Tuple[t.Union[str, int], ...] = ()
+        tmux_args: tuple[str | int, ...] = ()
 
         if g:
             tmux_args += ("-g",)
@@ -498,13 +551,11 @@ class Window(Obj):
 
         value_raw = next(shlex.split(item) for item in window_options_output)
 
-        value: t.Union[str, int] = (
-            int(value_raw[1]) if value_raw[1].isdigit() else value_raw[1]
-        )
+        value: str | int = int(value_raw[1]) if value_raw[1].isdigit() else value_raw[1]
 
         return value
 
-    def rename_window(self, new_name: str) -> "Window":
+    def rename_window(self, new_name: str) -> Window:
         """Rename window.
 
         Parameters
@@ -532,7 +583,7 @@ class Window(Obj):
             self.cmd("rename-window", new_name)
             self.window_name = new_name
         except Exception:
-            logger.exception(f"Error renaming window to {new_name}")
+            logger.exception("Error renaming window to %s", new_name)
 
         self.refresh()
 
@@ -540,7 +591,7 @@ class Window(Obj):
 
     def kill(
         self,
-        all_except: t.Optional[bool] = None,
+        all_except: bool | None = None,
     ) -> None:
         """Kill :class:`Window`.
 
@@ -578,7 +629,7 @@ class Window(Obj):
         >>> one_window_to_rule_them_all in session.windows
         True
         """
-        flags: t.Tuple[str, ...] = ()
+        flags: tuple[str, ...] = ()
 
         if all_except:
             flags += ("-a",)
@@ -591,13 +642,11 @@ class Window(Obj):
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
 
-        return None
-
     def move_window(
         self,
         destination: str = "",
-        session: t.Optional[str] = None,
-    ) -> "Window":
+        session: str | None = None,
+    ) -> Window:
         """Move current :class:`Window` object ``$ tmux move-window``.
 
         Parameters
@@ -628,15 +677,15 @@ class Window(Obj):
 
     def new_window(
         self,
-        window_name: t.Optional[str] = None,
+        window_name: str | None = None,
         *,
         start_directory: None = None,
         attach: bool = False,
         window_index: str = "",
-        window_shell: t.Optional[str] = None,
-        environment: t.Optional[t.Dict[str, str]] = None,
-        direction: t.Optional[WindowDirection] = None,
-    ) -> "Window":
+        window_shell: str | None = None,
+        environment: dict[str, str] | None = None,
+        direction: WindowDirection | None = None,
+    ) -> Window:
         """Create new window respective of current window's position.
 
         See Also
@@ -689,7 +738,7 @@ class Window(Obj):
     #
     # Climbers
     #
-    def select(self) -> "Window":
+    def select(self) -> Window:
         """Select window.
 
         To select a window object asynchrously. If a ``window`` object exists
@@ -725,7 +774,7 @@ class Window(Obj):
     # Computed properties
     #
     @property
-    def active_pane(self) -> t.Optional["Pane"]:
+    def active_pane(self) -> Pane | None:
         """Return attached :class:`Pane`."""
         panes = self.panes.filter(pane_active="1")
         if len(panes) > 0:
@@ -745,14 +794,14 @@ class Window(Obj):
         """Representation of :class:`Window` object."""
         return (
             f"{self.__class__.__name__}({self.window_id} "
-            + f"{self.window_index}:{self.window_name}, {self.session})"
+            f"{self.window_index}:{self.window_name}, {self.session})"
         )
 
     #
     # Aliases
     #
     @property
-    def id(self) -> t.Optional[str]:
+    def id(self) -> str | None:
         """Alias of :attr:`Window.window_id`.
 
         >>> window.id
@@ -764,7 +813,7 @@ class Window(Obj):
         return self.window_id
 
     @property
-    def name(self) -> t.Optional[str]:
+    def name(self) -> str | None:
         """Alias of :attr:`Window.window_name`.
 
         >>> window.name
@@ -776,7 +825,7 @@ class Window(Obj):
         return self.window_name
 
     @property
-    def index(self) -> t.Optional[str]:
+    def index(self) -> str | None:
         """Alias of :attr:`Window.window_index`.
 
         >>> window.index
@@ -788,7 +837,7 @@ class Window(Obj):
         return self.window_index
 
     @property
-    def height(self) -> t.Optional[str]:
+    def height(self) -> str | None:
         """Alias of :attr:`Window.window_height`.
 
         >>> window.height.isdigit()
@@ -800,7 +849,7 @@ class Window(Obj):
         return self.window_height
 
     @property
-    def width(self) -> t.Optional[str]:
+    def width(self) -> str | None:
         """Alias of :attr:`Window.window_width`.
 
         >>> window.width.isdigit()
@@ -816,15 +865,15 @@ class Window(Obj):
     #
     def split_window(
         self,
-        target: t.Optional[t.Union[int, str]] = None,
-        start_directory: t.Optional[str] = None,
+        target: int | str | None = None,
+        start_directory: StrPath | None = None,
         attach: bool = False,
         vertical: bool = True,
-        shell: t.Optional[str] = None,
-        size: t.Optional[t.Union[str, int]] = None,
-        percent: t.Optional[int] = None,  # deprecated
-        environment: t.Optional[t.Dict[str, str]] = None,
-    ) -> "Pane":
+        shell: str | None = None,
+        size: str | int | None = None,
+        percent: int | None = None,  # deprecated
+        environment: dict[str, str] | None = None,
+    ) -> Pane:
         """Split window and return the created :class:`Pane`.
 
         Notes
@@ -851,7 +900,7 @@ class Window(Obj):
             # Deprecated in 3.1 in favor of -l
             warnings.warn(
                 f'Deprecated in favor of size="{str(percent).rstrip("%")}%" '
-                + ' ("-l" flag) in tmux 3.1+.',
+                ' ("-l" flag) in tmux 3.1+.',
                 category=DeprecationWarning,
                 stacklevel=2,
             )
@@ -869,7 +918,7 @@ class Window(Obj):
         )
 
     @property
-    def attached_pane(self) -> t.Optional["Pane"]:
+    def attached_pane(self) -> Pane | None:
         """Return attached :class:`Pane`.
 
         Notes
@@ -888,7 +937,7 @@ class Window(Obj):
             return panes[0]
         return None
 
-    def select_window(self) -> "Window":
+    def select_window(self) -> Window:
         """Select window.
 
         Notes
@@ -924,7 +973,7 @@ class Window(Obj):
         if proc.stderr:
             raise exc.LibTmuxException(proc.stderr)
 
-    def get(self, key: str, default: t.Optional[t.Any] = None) -> t.Any:
+    def get(self, key: str, default: t.Any | None = None) -> t.Any:
         """Return key-based lookup. Deprecated by attributes.
 
         .. deprecated:: 0.16
@@ -934,7 +983,9 @@ class Window(Obj):
 
         """
         warnings.warn(
-            "Window.get() is deprecated", category=DeprecationWarning, stacklevel=2
+            "Window.get() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
         )
         return getattr(self, key, default)
 
@@ -954,7 +1005,7 @@ class Window(Obj):
         )
         return getattr(self, key)
 
-    def get_by_id(self, pane_id: str) -> t.Optional[Pane]:
+    def get_by_id(self, pane_id: str) -> Pane | None:
         """Return pane by id. Deprecated in favor of :meth:`.panes.get()`.
 
         .. deprecated:: 0.16
@@ -969,7 +1020,7 @@ class Window(Obj):
         )
         return self.panes.get(pane_id=pane_id, default=None)
 
-    def where(self, kwargs: t.Dict[str, t.Any]) -> t.List[Pane]:
+    def where(self, kwargs: dict[str, t.Any]) -> list[Pane]:
         """Filter through panes, return list of :class:`Pane`.
 
         .. deprecated:: 0.16
@@ -978,14 +1029,16 @@ class Window(Obj):
 
         """
         warnings.warn(
-            "Window.where() is deprecated", category=DeprecationWarning, stacklevel=2
+            "Window.where() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
         )
         try:
             return self.panes.filter(**kwargs)
         except IndexError:
             return []
 
-    def find_where(self, kwargs: t.Dict[str, t.Any]) -> t.Optional[Pane]:
+    def find_where(self, kwargs: dict[str, t.Any]) -> Pane | None:
         """Filter through panes, return first :class:`Pane`.
 
         .. deprecated:: 0.16
@@ -1000,7 +1053,7 @@ class Window(Obj):
         )
         return self.panes.get(default=None, **kwargs)
 
-    def _list_panes(self) -> t.List[PaneDict]:
+    def _list_panes(self) -> list[PaneDict]:
         """Return list of panes (deprecated in favor of :meth:`.panes`).
 
         .. deprecated:: 0.16
@@ -1016,7 +1069,7 @@ class Window(Obj):
         return [pane.__dict__ for pane in self.panes]
 
     @property
-    def _panes(self) -> t.List[PaneDict]:
+    def _panes(self) -> list[PaneDict]:
         """Property / alias to return :meth:`~._list_panes`.
 
         .. deprecated:: 0.16
@@ -1027,7 +1080,7 @@ class Window(Obj):
         warnings.warn("_panes is deprecated", category=DeprecationWarning, stacklevel=2)
         return self._list_panes()
 
-    def list_panes(self) -> t.List["Pane"]:
+    def list_panes(self) -> list[Pane]:
         """Return list of :class:`Pane` for the window.
 
         .. deprecated:: 0.16
@@ -1036,12 +1089,14 @@ class Window(Obj):
 
         """
         warnings.warn(
-            "list_panes() is deprecated", category=DeprecationWarning, stacklevel=2
+            "list_panes() is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
         )
         return self.panes
 
     @property
-    def children(self) -> QueryList["Pane"]:
+    def children(self) -> QueryList[Pane]:
         """Was used by TmuxRelationalObject (but that's longer used in this class).
 
         .. deprecated:: 0.16
@@ -1050,6 +1105,8 @@ class Window(Obj):
 
         """
         warnings.warn(
-            "Server.children is deprecated", category=DeprecationWarning, stacklevel=2
+            "Window.children is deprecated",
+            category=DeprecationWarning,
+            stacklevel=2,
         )
         return self.panes
